
/*
trayicon.exe - WIN32 application which resides in the system tray
Copyright (C) 2000  Antti Markus (antti@pld.ttu.ee)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __GNUC__
//useful defines
#define STRICT
#define WIN32_LEAN_AND_MEAN
#endif

//standard includes
#include <windows.h>

//this is not defined in mingw32 versions prior to 2.95
#ifndef TPM_BOTTOMALIGN
#define TPM_BOTTOMALIGN     0x0020L
#endif

//additional includes
#include "trayicon.h"
#include "resource.h"

//trayicon notification message
#define WM_TRAY   WM_USER+0x100+5

LRESULT CALLBACK WindowFunc(HWND, UINT, WPARAM, LPARAM);

const char* pClassName="TTWin32TrayApp123"; //window class name
HINSTANCE hMainInst=0;    //application main instance handle
HICON hIcon1=0, hIcon2=0; //icon handles
unsigned uWhich=0;        //indicates which icon is displayed
HMENU hPopup=0;           //menu handle
CTrayIcon* pTrayIcon=0;   //pointer to trayicon class



int WINAPI WinMain(HINSTANCE hInstance,
                   HINSTANCE hPrevInstance,
                   LPSTR lpszArgs,
                   int nWinMode)
{
  WNDCLASS WinClass;

  //register window class
  WinClass.hInstance=hInstance;
  hMainInst=hInstance;
  WinClass.lpszClassName=pClassName;  //specify class name
  WinClass.lpfnWndProc=WindowFunc;    //specify callback function
  WinClass.style=0;
  WinClass.hIcon=LoadIcon(NULL, IDI_APPLICATION); //specify icon
  WinClass.hCursor=LoadCursor(NULL, IDC_ARROW);   //specify cursor
  WinClass.lpszMenuName=0;
  WinClass.cbClsExtra=0;
  WinClass.cbWndExtra=0;
  WinClass.hbrBackground=(HBRUSH)GetStockObject(WHITE_BRUSH); //specify window background

  if(!RegisterClass(&WinClass))
    return 0;

  //create window
  HWND hwnd=CreateWindow(pClassName,
                         0,
                         WS_OVERLAPPEDWINDOW,
                         10,
                         10,
                         10,
                         10,
                         HWND_DESKTOP,
                         NULL,
                         hInstance,
                         NULL);

  //hide window
  ShowWindow(hwnd, SW_HIDE);

  //start message pump
  MSG WinMsg;
  while(GetMessage(&WinMsg, NULL, 0, 0))
  {
    TranslateMessage(&WinMsg);
    DispatchMessage(&WinMsg);
  }

  return WinMsg.wParam;
}

LRESULT CALLBACK WindowFunc(HWND hwnd, UINT Msg,
                            WPARAM wParam, LPARAM lParam)
{
  switch(Msg)
  {
    //application starts up
    case WM_CREATE:
    {
      //initialize icons
      hIcon1=LoadIcon(hMainInst, MAKEINTRESOURCE(IDI_CHECK));
      hIcon2=LoadIcon(hMainInst, MAKEINTRESOURCE(IDI_CROSS));
      //initialize popup menu
      hPopup=LoadMenu(hMainInst, MAKEINTRESOURCE(IDR_MENU));

      //initialize tray icon
      pTrayIcon=new CTrayIcon(hwnd, WM_TRAY, IDI_CHECK, hIcon1, "Checked");
      //put icon to system tray
      pTrayIcon->Put();
      break;
    }

    //notification from trayicon received
    case WM_TRAY:
    {
      //if right mouse button pressed
      if(lParam == WM_RBUTTONDOWN)
      {
        POINT pt;
        GetCursorPos(&pt);  //get cursor position
        SetForegroundWindow(hwnd);  //set window to foreground
        //display popup menu
        BOOL bRes=TrackPopupMenu(GetSubMenu(hPopup, 0), TPM_BOTTOMALIGN | TPM_RIGHTALIGN,
                                 pt.x, pt.y, 0, hwnd, 0);
        //send dummy message to window
        SendMessage(hwnd, WM_NULL, 0, 0);
      }
      break;
    }

    //notification from menu received
    case WM_COMMAND:
    {
      switch(LOWORD(wParam))
      {
        case IDM_EXIT:
        {
          DestroyWindow(hwnd);
          break;
        }

        //change icon in system tray
        case IDM_CHANGE:
        {
          if(!uWhich)
          {
            uWhich=1;
            pTrayIcon->Modify(hIcon2);
            pTrayIcon->Modify("Crossed");
          }
          else
          {
            uWhich=0;
            pTrayIcon->Modify(hIcon1);
            pTrayIcon->Modify("Checked");
          }
          break;
        }
      }
      break;
    }

    //exit application
    case WM_DESTROY:
    {
      //release used resources
      delete pTrayIcon;
      DestroyIcon(hIcon1);
      DestroyIcon(hIcon2);
      DestroyMenu(hPopup);
      PostQuitMessage(0);
      break;
    }

    default:
      return DefWindowProc(hwnd, Msg, wParam, lParam);
  }
  return 0;
}